/*
 * klibc/seek.h
 *
 * SEEK constants - needed by stdio.h, fcntl.h, and unistd.h
 */

#ifndef _KLIBC_SEEK_H
#define _KLIBC_SEEK_H

#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2

#endif /* _KLIBC_SEEK_H */
